/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.client;

import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.Timer;
import java.util.TimerTask;

import de.bsvrz.dav.daf.main.ApplicationCloseActionHandler;
import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientDavParameters;
import de.bsvrz.dav.daf.main.CommunicationError;
import de.bsvrz.dav.daf.main.ConnectionException;
import de.bsvrz.dav.daf.main.DavConnectionListener;
import de.bsvrz.dav.daf.main.InconsistentLoginException;
import de.bsvrz.dav.daf.main.MissingParameterException;
import de.bsvrz.dav.daf.main.config.ConfigurationArea;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.ObjectTimeSpecification;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.ibv.uda.interpreter.UdaInterpreter;
import de.bsvrz.ibv.uda.interpreter.UdaInterpreterException;
import de.bsvrz.ibv.uda.interpreter.UdaParserFehler;
import de.bsvrz.ibv.uda.uda.data.SkriptLauf;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Realisierung einer logsichen Verbindung mit einer Anwendung "Umfassende
 * Datenanalyse"<br>
 * Die Klasse stellt eine neue Datenverteilerverbindung auf Basis der
 * übergebenen Verbindung her und führt eine Anmeldung mit den gegebenen
 * Nutzerdaten aus. Die Anmeldeversuche werden solange wiederholt, bis eine
 * Verbindung hergestellt werden konnte bzw. wir automatisch wiederhergestellt,
 * wenn die Verbindung unterbrochen wird. Innerhalb der Klasse werden die Daten
 * der über eine Liste von PID definierten UDA-Module verwaltet und zur
 * Verfügung gestellt.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class UdaClient extends TimerTask implements DavConnectionListener, ApplicationCloseActionHandler {

	private static final Debug LOGGER = Debug.getLogger();

	/**
	 * die Verbindung zum Datenverteiler, über die die Klasse mit der
	 * "Umfassenden Datenanalyse" kommuniziert.
	 */
	private ClientDavInterface verbindung;

	/**
	 * Marker für die Abbildung des Verbindungszustandes.
	 */
	private boolean connected;

	/**
	 * Timer für die wiederholte Verbindungsherstellung beim Start ohne
	 * Datenverteiler oder beim Abbruch der Verbindung.
	 */
	private final Timer timer = new Timer("Datenverteiler-Verbinder");

	/**
	 * die Listener, die bei Änderung des Verbindungsstatus benachrichtigt
	 * werden.
	 */
	private final Set<UdaConnectionListener> connectionListeners = new HashSet<>();

	/**
	 * der Name, mit dem eine Anmeldung beim Datenverteiler erfolgen soll.
	 */
	private final String nutzer;

	/**
	 * das Passwort, des Nutzers mit dem die Anmeldung erfolgen soll.
	 */
	private final String passwort;

	/**
	 * die Liste der UdaModule, die von der Klasse verwaltet werden.
	 */
	private final Map<ConfigurationObject, ClientModul> module = new HashMap<>();

	/**
	 * die Liste der PIDs, mit denen die zu verwaltenden UdaModule beschrieben
	 * werden.
	 */
	private final String[] modulPids;

	/**
	 * Konstruktor.<br>
	 * Die Funktion erzeugt eine Instanz der Klasse, die eine
	 * Datenverteilerverbindung mit den Parametern der übergebenen Verbindung
	 * herstellt. Wird keine Verbindung übergeben, werden die Standard-Parameter
	 * für eine Datenverteilerverbindung verwendet.<br>
	 * Die Anmeldung beim Datenverteiler erfolgt mit den übergebenen
	 * Identifikationsdaten.<br>
	 * Die Liste der PIDs beschreibt, die zu verwaltenden Module.
	 *
	 * @param verbindung
	 *            die Datenverteilerverbindung
	 * @param nutzer
	 *            der Name des Nutzers für die Datenverteileranmeldung
	 * @param passwort
	 *            das Passwort für die Anmeldung
	 * @param modulPids
	 *            die Liste der PIDs zur Ermittlung der UdaModule
	 * @throws MissingParameterException
	 *             ein für die Herstellung der Verbindung erforderlicher
	 *             Parameter wurde nicht angegeben
	 */
	public UdaClient(final ClientDavInterface verbindung, final String nutzer, final String passwort,
			final String[] modulPids) throws MissingParameterException {

		this.modulPids = modulPids;
		if (verbindung == null) {
			this.verbindung = new ClientDavConnection(new ClientDavParameters());
		} else {
			this.verbindung = new ClientDavConnection(verbindung.getClientDavParameters());
		}

		this.nutzer = nutzer;
		this.passwort = passwort;

		verbinden();

		if (nutzer != null) {
			this.verbindung.addConnectionListener(this);
			this.verbindung.setCloseHandler(this);
			timer.schedule(this, 1L, 1000L);
		}
	}

	/**
	 * die Funktion stellt die Verbindung zum Datenverteiler mit den definierten
	 * Parametern her.<br>
	 * Wenn eine Verbindung hergestellt bzw. wiederhergestellt werden konnte,
	 * werden alle Listener, die sich für den Verbindungsstatus angemeldet haben
	 * benachrichtigt.
	 */
	private void verbinden() {
		if (!connected) {
			synchronized (module) {
				try {
					verbindung.connect();
					verbindung.login(nutzer, passwort);
					connected = true;
					initialisiereModule();
					synchronized (connectionListeners) {
						for (final UdaConnectionListener listener : connectionListeners) {
							listener.connected();
						}
					}

					// Alle Exceptions beim Herstellen der Verbindung werden
					// ignoriert, da der Verbindungsversuch zyklisch wiederholt
					// wird.

				} catch (final CommunicationError | ConnectionException | InconsistentLoginException e) {
					LOGGER.warning(e.getLocalizedMessage());
					connected = false;
				} catch (final RuntimeException e) {
					verbindung.disconnect(false, e.getLocalizedMessage());
					connected = false;
				}
			}
		}
	}

	/**
	 * liefert die Verbindung, über den das Modul mit dem Datenverteiler
	 * kommunizieren sollen.
	 *
	 * @return die Verbindung
	 */
	public ClientDavInterface getVerbindung() {
		return verbindung;
	}

	/**
	 * ermittelt, ob die Verbindung des Moduls mit dem Datenverteiler
	 * erfolgreich hergestellt werden konnte.
	 *
	 * @return <i>true</i>, wenn eine gültige Verbindung besteht
	 */
	public boolean isConnected() {
		return connected;
	}

	/**
	 * {@inheritDoc}. Die Funktion versucht die Verbindung zum Datenverteiler
	 * bei Bedarf wiederherzustellen.
	 *
	 * @see java.util.TimerTask#run()
	 */
	@Override
	public void run() {
		verbinden();
	}

	/**
	 * fügt der Klasse einen Listener hinzu, der bei der Änderung des
	 * Verbindungszustandes benachrichtigt werden soll.
	 *
	 * @param listener
	 *            der hinzuzufügende Listener
	 * @see #removeConnectionListener(UdaConnectionListener)
	 */
	public void addConnectionListener(final UdaConnectionListener listener) {
		connectionListeners.add(listener);
		verbindung.addConnectionListener(listener);
	}

	/**
	 * entfernt einen Listener, der bei der Änderung des Verbindungszustandes
	 * benachrichtigt wurde.
	 *
	 * @param listener
	 *            der zu entfernende Listener
	 * @see #addConnectionListener(UdaConnectionListener)
	 */
	public void removeConnectionListener(final UdaConnectionListener listener) {
		connectionListeners.remove(listener);
		verbindung.removeConnectionListener(listener);
	}

	/**
	 * behandelt den Abbruch der Verbindung zum Datenverteiler.<br>
	 * Die Liste der Module wird zurückgesetzt. {@inheritDoc}.
	 *
	 * @see de.bsvrz.dav.daf.main.DavConnectionListener#connectionClosed(de.bsvrz.dav.daf.main.ClientDavInterface)
	 */
	@Override
	public void connectionClosed(final ClientDavInterface conn) {
		connected = false;
		initialisiereModule();
	}

	/**
	 * {@inheritDoc}. Die Funktion implementiert die entsprechende Funktion des
	 * CloseHandlers, um ein Beenden der gesamten Anwendung zu verhindern
	 * (Standardverhalten).<br>
	 * Die Liste der Module wird zurückgesetzt.
	 *
	 * @see de.bsvrz.dav.daf.main.ApplicationCloseActionHandler#close(java.lang.String)
	 */
	@Override
	public void close(final String error) {
		connected = false;
		initialisiereModule();
	}

	/**
	 * initialisiert die Liste der verwalteten Module.<br>
	 * Die Funktion wird beim Herstellen bzw. Trennen der
	 * Datenverteilerverbindung aufgerufen.<br>
	 * Beim Verbindungsabbruch wird die Liste der verwalteten UdaModule geleert.
	 * <br>
	 * Bei bestehender Verbindung werden auf Basis der an den Konstruktor
	 * übergebenen Liste der PIDs die zu verwaltenden UdaModule ermittelt und
	 * initialisiert.<br>
	 * Aus den PIDs werden die entsprechenden Systemobjekte ermittelt und wie
	 * folgt verwendet.
	 * <ul>
	 * <li>das Objekt ist selbst ein UdaModul: das entsprechende Modul wird
	 * Bestandteil der Liste der verwalteten Module</li>
	 * <li>das Objekt ist ein Konfigurationsbereich: es werden alle UdaModule,
	 * die innerhalb des angegebenen Konfigurationsbereches definiert sind
	 * aufgenommen</li>
	 * </ul>
	 * Wenn die Liste der PIDs leer ist, werden alle Objekte vom Typ UdaModul in
	 * die Verwaltung aufgenommen.
	 */
	private void initialisiereModule() {
		if (!isConnected()) {
			for (final ClientModul client : module.values()) {
				client.clear();
			}
			module.clear();
		} else {
			final Set<ConfigurationObject> modulObjekte = new HashSet<>();

			if ((modulPids == null) || (modulPids.length == 0)) {
				final SystemObjectType type = verbindung.getDataModel().getType("typ.udaModul");
				for (final SystemObject modul : type.getElements()) {
					modulObjekte.add((ConfigurationObject) modul);
				}
			} else {
				final SystemObjectType modulTyp = verbindung.getDataModel().getType("typ.udaModul");

				for (final String pid : modulPids) {
					final SystemObject obj = verbindung.getDataModel().getObject(pid);
					if (obj instanceof ConfigurationArea) {
						for (final SystemObject modul : ((ConfigurationArea) obj).getObjects(
								Arrays.asList(new SystemObjectType[] { modulTyp }), ObjectTimeSpecification.valid())) {
							modulObjekte.add((ConfigurationObject) modul);
						}
					} else if (obj.isOfType(modulTyp)) {
						modulObjekte.add((ConfigurationObject) obj);
					}
				}
			}

			for (final ConfigurationObject modul : modulObjekte) {
				final ClientModul client = new ClientModul(verbindung, modul);
				module.put(modul, client);
			}
		}
	}

	/**
	 * liefert die Menge der verwalteten UdaModule.
	 *
	 * @return die Module
	 */
	public Collection<ClientModul> getModule() {
		synchronized (module) {
			return module.values();
		}
	}

	/**
	 * prüfen eines Uda-Skripttextes auf syntaktische Korrektheit.<br>
	 *
	 * @param fileName
	 *            der Name der Datei, die den Skripttext enthält
	 * @return das Ergebnis
	 */
	@SuppressWarnings("unused")
	public ParserResult parseSkriptText(final String fileName) {
		final ParserResult result = new ParserResult();
		final SkriptLauf lauf = new SkriptLaufDaten(null, "",
				new LokalesSkript(fileName));
		try {
			new UdaInterpreter(null, lauf);
		} catch (final UdaInterpreterException e) {
			for (final UdaParserFehler fehler : e.getFehlerListe()) {
				result.addFehler(fehler);
			}
		}

		return result;
	}
}
