/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import de.bsvrz.ibv.uda.interpreter.anweisung.AbstractAnweisung;
import de.bsvrz.ibv.uda.interpreter.anweisung.Anweisung;
import de.bsvrz.ibv.uda.interpreter.anweisung.MitFehlerRueckgabeAnweisung;
import de.bsvrz.ibv.uda.interpreter.anweisung.RueckgabeAnweisung;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.UdaFehler;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Kontext;
import de.bsvrz.sys.funclib.bitctrl.interpreter.NichtWert;

/**
 * Ein Block von Anweisungen in einem Uda-Skript.<br>
 * EIn Anweisungsblock setzt sich aus einer Liste von Anweisungen zusammen, die
 * nacheinander ausgef&uuml;hrt werden. Ein Skript besteht im wesentlichen
 * selbst aus einem Anweisungsblock. Anweisungsbl&ouml;cke k&ouml;nnen aber auch
 * Bestandteile von anderen strukturierten Anweisungen sein.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class AnweisungsBlock extends AbstractAnweisung {

	/**
	 * definiert, ob die Ausführung des Anweisungsblocks aktiv ist, d.h. ob die
	 * nächstfolgende Anweisung ausgeführt werden soll.
	 */
	private Boolean running = true;

	/**
	 * definiert, ob die Ausführung des Anweisungsblockes nach Ausführung der
	 * aktuellen Anweisung abgebrochen werden soll.
	 */
	private boolean abbruch;

	/**
	 * die Liste der Anweisungen des Blockes.
	 */
	private final List<Anweisung> anweisungsListe = new ArrayList<>();

	/**
	 * Konstruktor.
	 *
	 * @param zeile
	 *            die Nummer der Zeile, in der der Anweisungsblock im Quelltext
	 *            definiert ist
	 * @param skript
	 *            der Name des Skripts
	 */
	public AnweisungsBlock(final int zeile, final String skript) {
		super(zeile, skript);
	}

	/**
	 * f&uuml;gt dem Anweisungsblock eine Anweisung hinzu.<br>
	 * Die Funktion wird vom Interpreter beim &Uuml;bersetzen des vom Parser
	 * gelieferten AST in den Interpreterbaum aufgerufen.
	 *
	 * @param anweisung
	 *            die einzuf&uuml;gende Anweisung
	 */
	public void addAnweisung(final Ausdruck anweisung) {
		if (anweisung instanceof Anweisung) {
			anweisungsListe.add((Anweisung) anweisung);
		}
	}

	/**
	 * fügt der Anweisungsliste die übergebene Liste von Anweisungen hinzu.
	 *
	 * @param anweisungen
	 *            eine Liste von Anweisungen
	 */
	protected void addAnweisungen(final Collection<Anweisung> anweisungen) {
		anweisungsListe.addAll(anweisungen);
	}

	/**
	 * führt die Anweisungen des Anweisungsblockes aus.<br>
	 * Die Funktion wird intern von den <i>interpret</i> und <i>fortsetzen</i>
	 * aufgerufen.
	 *
	 * @param kontext
	 *            der Kontext, mit dem die Anweisungen ausgewertet werden
	 * @return das Ergebnis der Auswertung der Anweisungen
	 */
	private Object anweisungenAusfuehren(final Kontext kontext) {
		Object result = null;

		for (final Anweisung anweisung : anweisungsListe) {
			while (!running) {
				try {
					synchronized (this) {
						wait();
					}
				} catch (final InterruptedException e) {
					e.printStackTrace();
				}
			}
			if (abbruch) {
				break;
			}
			result = anweisung.interpret(kontext);
			if ((anweisung instanceof RueckgabeAnweisung) || (result instanceof RueckgabeAnweisung)) {
				break;
			}
			if (anweisung instanceof MitFehlerRueckgabeAnweisung) {
				if (result instanceof UdaFehler) {
					break;
				}
			}
		}

		if (result == null) {
			result = NichtWert.NICHTWERT;
		}

		return result;
	}

	/**
	 * beendet die Ausführung des Anweisungsblock nach dem Ende der Ausführung
	 * der aktuellen Anweisung.
	 */
	public void beenden() {
		synchronized (this) {
			running = true;
			abbruch = true;
			notify();
		}
	}

	/**
	 * liefert die Liste der Anweisungen.
	 *
	 * @return die Liste
	 */
	List<Anweisung> getAnweisungen() {
		return anweisungsListe;
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck#getNachfolger()
	 */
	@Override
	public List<Ausdruck> getNachfolger() {
		return new ArrayList<>(anweisungsListe);
	}

	/**
	 * Auswertung der Anweisungen des Anweisungsblockes.<br>
	 * Die Funktion f&uuml;hrt nacheinander alle Anweisungen die in dem Block
	 * enthalten sind aus. Das Ergebnis der Auswertung ist das Ergebnis der
	 * zuletzt ausgef&uuml;hrten Anweisungen. Das ist in der Regel eine
	 * Rückgabeanweisung.<br>
	 * Tritt bei der Ausf&uuml;hrung des Anweisungsblocks ein Fehler auf, der
	 * nicht innerhalb der betroffenen Anweisung abgefangen werden konnte, wird
	 * der aufgetretene Fehler als Ausnahme geworfen.<br>
	 * Wird von den ausgef&uuml;hrten Anweisungen kein Ergebnis ermittelt,
	 * liefert die Funktion den Wert "undefiniert".<br>
	 *
	 * {@inheritDoc}
	 *
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck#interpret(de.bsvrz.sys.funclib.bitctrl.interpreter.Kontext)
	 */
	@Override
	public Object interpret(final Kontext kontext) {
		try {
			return anweisungenAusfuehren(kontext);
		} catch (final RuntimeException e) {
			throw erzeugeAnweisungsFehler(e);
		}
	}

	/**
	 * setzt den Status für die Ausführung der nachfolgenden Anweisungen.
	 *
	 * @param status
	 *            der Status
	 */
	protected void setRunning(final boolean status) {
		synchronized (this) {
			running = status;
			notify();
		}
	}
}
