/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.anweisung;

import java.io.File;

import de.bsvrz.ibv.uda.interpreter.daten.SkriptKontext;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.Fehler;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.UdaFehler;
import de.bsvrz.ibv.uda.verwaltung.protokoll.ProtokollEintrag;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck;
import de.bsvrz.sys.funclib.bitctrl.interpreter.InterpreterException;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Kontext;
import de.bsvrz.sys.funclib.bitctrl.interpreter.NichtWert;

/**
 * die Repräsentation einer Ausgabeanweisung innerhalb des Uda-Interpreters.<br>
 * Eine Ausgabeanweisung ist eine "einfache Anweisung", d.h. sie enthält nur
 * einen einzigen Ausdruck. Der Audruck wird ausgewertet und das Ergebnis der
 * Auswertung wird über die innerhalb des Interpreters definierten Ausgabekanäle
 * ausgegeben.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class AusgabeAnweisung extends AbstractEinfacheAnweisung {

	/**
	 * wenn der Wert auf <i>true</i> steht, wird bei der Ausgabe der Name des
	 * Skripts und die Zeilennummer der Anweisung mit ausgegeben.
	 */
	private final boolean debug;

	/**
	 * Konstruktor.
	 *
	 * @param zeile
	 *            die Zeile innerhalb des Quelltextes, in dem die Anweisung
	 *            definiert ist
	 * @param skriptname
	 *            der Name des Skripts
	 * @param debug
	 *            erweiterte Debug-Ausgabe ?
	 * @param ausdruck
	 *            der auszuwertende Ausdruck
	 */
	public AusgabeAnweisung(final int zeile, final String skriptname, final boolean debug, final Ausdruck ausdruck) {
		super(zeile, skriptname, ausdruck);
		assert ausdruck != null;
		this.debug = debug;
	}

	/**
	 * Die Funktion führt die Ausgabeanweisung aus.<br>
	 * Bei der Ausführung der Funktion wird der enthaltene Ausdruck ausgewertet,
	 * der Wert des Ausdrucks wird in eine Zeichenkette konvertiert und über die
	 * Ausgabekanäle des Interpreters ausgegeben. Treten bei der Auswertung des
	 * zugehörigen Ausdrucks Fehler auf, wird als stellvertretendes Ergebnis des
	 * Ausdruck ein UdaFehler erzeugt. Vom Ausdruck selbst gelieferte
	 * Fehlerobjekte werden um die Informationen zu Skriptname und Zeilennummer
	 * ergänzt.<br>
	 * {@inheritDoc}
	 *
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck#interpret(de.bsvrz.sys.funclib.bitctrl.interpreter.Kontext)
	 */
	@Override
	public Object interpret(final Kontext kontext) {
		final StringBuffer buffer = new StringBuffer();
		if (debug) {
			buffer.append(new File(getSkript()).getName());
			buffer.append(" (");
			buffer.append(getZeile());
			buffer.append("): ");
		}

		final Ausdruck ausdruck = getAusdruck();
		Object value = null;
		try {
			value = ausdruck.interpret(kontext);
			if (value instanceof UdaFehler) {
				value = erzeugeAnweisungsFehler((UdaFehler) value);
			}
		} catch (final InterpreterException e) {
			value = erzeugeAnweisungsFehler(e);
			// throw (UdaFehler) value;
		}
		if (value == null) {
			value = new Fehler();
		}
		buffer.append(value.toString());
		((SkriptKontext) kontext).getInterpreter().ausgabe(new ProtokollEintrag(buffer.toString()));

		return NichtWert.NICHTWERT;
	}

}
