/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.anweisung;

import de.bsvrz.ibv.uda.interpreter.daten.fehler.Fehler;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.UdaFehler;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck;
import de.bsvrz.sys.funclib.bitctrl.interpreter.InterpreterException;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Kontext;

/**
 * die Repräsentation einer Rückgabeanweisung in einem Uda-Skript.<br>
 * Eine Rückgabeanweisung liefert den Wert des zugeordneten Ausdrucks zurück.
 * Der Anweisungsblock in dem die Anweisung ausgeführt wurde wird beendet. Diese
 * Funktionalität ist innerhalb des Anweisungsblocks implementiert und nicht
 * Bestandteil der Funktionalität dieser Klasse.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class RueckgabeAnweisung extends AbstractEinfacheAnweisung {

	/**
	 * der ermittelte Rückgabewert.
	 */
	private Object rueckgabeWert;

	/**
	 * Konstruktor.
	 *
	 * @param zeile
	 *            die Nummer der Zeile, in der die Anweisung definiert ist
	 * @param skriptname
	 *            der Name des Skripts
	 * @param ausdruck
	 *            der auszuwertende Ausdruck
	 */
	public RueckgabeAnweisung(final int zeile, final String skriptname, final Ausdruck ausdruck) {
		super(zeile, skriptname, ausdruck);
	}

	/**
	 * liefert den Rückgabewert der Anweisung, der bei der letzten Auswerting
	 * ermittelt wurde.
	 *
	 * @return den Wert
	 */
	public Object getRueckgabeWert() {
		if (rueckgabeWert == null) {
			throw new Fehler("Der Ausdruck der Anweisung wurde nicht ausgewertet");
		}
		return rueckgabeWert;
	}

	/**
	 * führt die Anweisung aus.<br>
	 * Zur Ausführung der Anweisung wird der zugeordnete Ausdruck ausgewertet.
	 * Das Ergebnis der Auswertung ist die Rückgabeanweisung selbst, um bei
	 * verschachtelten Anweisungsblöcken einen Funktionrückgabewert durchreichen
	 * zu können. Das Ergebnis der Auswerting kann mit der Funktion
	 * {@link RueckgabeAnweisung#getRueckgabeWert()} ermittelt werden.<br>
	 * Wird bei der Auswertung des Ausdrucks eine Ausnahme nicht abgefangen,
	 * wird ein entsprechendes Fehlerobjekt erzeugt und als Ergebnis der
	 * Anweisung zurückgeliefert.<br>
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck#interpret(de.bsvrz.sys.funclib.bitctrl.interpreter.Kontext)
	 */
	@Override
	public Object interpret(final Kontext kontext) {
		rueckgabeWert = null;
		try {
			rueckgabeWert = getAusdruck().interpret(kontext);
			if (rueckgabeWert instanceof UdaFehler) {
				rueckgabeWert = erzeugeAnweisungsFehler((UdaFehler) rueckgabeWert);
			}
		} catch (final InterpreterException e) {
			throw erzeugeAnweisungsFehler(e);
		}
		return this;
	}
}
