/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.anweisung.anmeldung;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import de.bsvrz.ibv.uda.interpreter.UdaInterpreter;
import de.bsvrz.ibv.uda.interpreter.daten.container.UdaContainer;
import de.bsvrz.ibv.uda.interpreter.daten.container.UdaIterator;
import de.bsvrz.ibv.uda.interpreter.daten.dav.MengenObjekt;
import de.bsvrz.ibv.uda.interpreter.daten.zeit.ZeitDauer;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Kontext;

/**
 * die Repräsentation eines Ausdrucks zur Definition einer Anmeldung zum
 * Empfang/Versenden von Daten.<br>
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class AtgAnmeldungsEintragAusdruck implements AnmeldeAusdruck {

	/**
	 * der Ausdruck zur Beschreibung der anzumeldenen Systemobjekte.
	 */
	private final Ausdruck objektAusdruck;

	/**
	 * der Ausdruck zur Beschreibung der Attributgruppe.
	 */
	private final Ausdruck atgAusdruck;

	/**
	 * der Ausdruck zur Beschreibung des Aspekts.
	 */
	private final Ausdruck aspAusdruck;

	/**
	 * der Ausdruck zur Definition eines Timeoutwertes.
	 */
	private final Ausdruck zeitAusdruck;

	/**
	 * der Interpreter der den Ausdruck auswerten soll.
	 */
	private final UdaInterpreter interpreter;

	/**
	 * Konstruktor.
	 *
	 * @param interpreter
	 *            der auswertende Interpreter
	 * @param objekte
	 *            der Ausdruck zur Beschreibung der Systemobjekte
	 * @param atg
	 *            der Ausdruck zur Beschreibung der Attributgruppe
	 * @param asp
	 *            der Ausdruck zur Beschreibung des Aspekts
	 * @param zeit
	 *            der Ausdruck zur Beschreibung eines Timeoutwertes (optional)
	 */
	public AtgAnmeldungsEintragAusdruck(final UdaInterpreter interpreter, final Ausdruck objekte, final Ausdruck atg,
			final Ausdruck asp, final Ausdruck zeit) {
		this.interpreter = interpreter;
		objektAusdruck = objekte;
		atgAusdruck = atg;
		aspAusdruck = asp;
		zeitAusdruck = zeit;
	}

	/**
	 * Die Funktion ermittelt aus dem an die Klasse übergeben Objektausdruck
	 * alle Objekte die Systemobjekte repräsentieren, sowie die Objekte für die
	 * Beschreibung der Attributgruppe, des Aspekts und des optionalen
	 * Timeoutwertes. Für jedes ermittelte Systemobjekt wird ein Anmeldeeintrag
	 * erzeugt. Als Ergebnis wird die Liste der erzeugten Einträge geliefert.
	 * Tritt beim Anlegen des Eintrags ein unbehandelter Fehler auf, wird der
	 * Eintrag übergangen.
	 *
	 * @param kontext
	 *            der Kontext in dem der Ausdruck ausgewertet wird
	 * @return liefert die Liste der Anmeldungseinträge, die sich aus dem
	 *         Ausdruck ergeben
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck#interpret(de.bsvrz.sys.funclib.bitctrl.interpreter.Kontext)
	 */
	@Override
	public AtgAnmeldungsEintrag[] interpret(final Kontext kontext) {
		final Collection<AtgAnmeldungsEintrag> anmeldungen = new ArrayList<>();
		final Object objekte = objektAusdruck.interpret(kontext);
		final Object atg = atgAusdruck.interpret(kontext);
		final Object asp = aspAusdruck.interpret(kontext);
		ZeitDauer dauer = null;
		if (zeitAusdruck != null) {
			dauer = (ZeitDauer) zeitAusdruck.interpret(kontext);
		}
		if (objekte instanceof UdaContainer) {
			final UdaIterator iterator = ((UdaContainer) objekte).getUdaIterator(UdaIterator.ANFANG);
			while (iterator.weiter().getBoolWert()) {
				anmeldungen.add(
						new AtgAnmeldungsEintrag(interpreter.getConnection(), iterator.getElement(), atg, asp, dauer));
			}
		} else if (objekte instanceof MengenObjekt) {
			final UdaIterator iterator = ((MengenObjekt) objekte).getMengenObjekte().getUdaIterator(UdaIterator.ANFANG);
			while (iterator.weiter().getBoolWert()) {
				anmeldungen.add(
						new AtgAnmeldungsEintrag(interpreter.getConnection(), iterator.getElement(), atg, asp, dauer));
			}
		} else {
			anmeldungen.add(new AtgAnmeldungsEintrag(interpreter.getConnection(), objekte, atg, asp, dauer));
		}
		return anmeldungen.toArray(new AtgAnmeldungsEintrag[0]);
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck#getNachfolger()
	 */
	@Override
	public List<Ausdruck> getNachfolger() {
		final List<Ausdruck> liste = new ArrayList<>();
		if (objektAusdruck != null) {
			liste.add(objektAusdruck);
		}
		if (atgAusdruck != null) {
			liste.add(atgAusdruck);
		}
		if (aspAusdruck != null) {
			liste.add(aspAusdruck);
		}
		if (zeitAusdruck != null) {
			liste.add(zeitAusdruck);
		}
		return liste;
	}
}
