/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.ausdruck;

import java.util.ArrayList;
import java.util.List;

import de.bsvrz.ibv.uda.interpreter.anweisung.RueckgabeAnweisung;
import de.bsvrz.ibv.uda.interpreter.daten.SkriptKontext;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.ArgumentFehler;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.SymbolUndefiniertFehler;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.UdaFehler;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.UdaFehlerSubtyp;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Kontext;
import de.bsvrz.sys.funclib.bitctrl.interpreter.NichtWert;
import de.bsvrz.sys.funclib.bitctrl.interpreter.OperationsSymbol;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Operator;

/**
 * Repräsentation eines Ausdrucks für einen Funktionsaufruf.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class FunktionsAufruf implements Ausdruck {

	/**
	 * der Name der Funktion.
	 */
	private final String name;

	/**
	 * die Ausdrücke, die die Argumente der Funktion beschreiben.
	 */
	private final List<Ausdruck> argumente = new ArrayList<>();

	/**
	 * Konstruktor.<br>
	 *
	 * @param name
	 *            der Name der aufzurufenden Funktion
	 * @param argumente
	 *            die Ausdrücke zur Beschreibung der zu übergebenden Parameter
	 */
	public FunktionsAufruf(final String name, final List<Ausdruck> argumente) {
		this.name = name;
		this.argumente.addAll(argumente);
	}

	/**
	 * liefert die Liste der Ausdrücke, die die Argumente des Funktionsaufrufs
	 * beschreiben.
	 *
	 * @return die Liste der Ausdrücke
	 */
	public List<Ausdruck> getArgumente() {
		return argumente;
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck#getNachfolger()
	 */
	@Override
	public List<Ausdruck> getNachfolger() {
		List<Ausdruck> liste = null;
		if (argumente.size() > 0) {
			liste = new ArrayList<>(argumente);
		}
		return liste;
	}

	/**
	 * liefert den Namen der aufzurufenden Funktion.
	 *
	 * @return den Name
	 */
	public String getName() {
		return name;
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck#interpret(de.bsvrz.sys.funclib.bitctrl.interpreter.Kontext)
	 */
	@Override
	public Object interpret(final Kontext kontext) {
		Object result = null;
		FunktionsDefinition definition = null;

		if (kontext instanceof SkriptKontext) {
			definition = ((SkriptKontext) kontext).getFunktion(name);
			if (definition == null) {
				definition = ((SkriptKontext) kontext).getInterpreter().getFunktion(name);
			}
			if (definition != null) {
				final Kontext lokalKontext = new SkriptKontext((SkriptKontext) kontext, false);

				if (definition.getArgumente().size() != argumente.size()) {
					throw new ArgumentFehler(UdaFehlerSubtyp.ANZAHL,
							"Die Anzahl der Argumente für den Aufruf der Funktion \""
									+ getName() + "\" ist nicht korrekt");
				}

				for (int idx = 0; (idx < (definition).getArgumente().size()) && (idx < argumente.size()); idx++) {
					lokalKontext.set((definition).getArgumente().get(idx), argumente.get(idx).interpret(kontext));
				}
				result = definition.interpret(lokalKontext);
			} else {
				if (Operator.enthaelt(name)) {
					result = new OperationsSymbol(Operator.getOperator(name), argumente).interpret(kontext);
				} else {
					throw new SymbolUndefiniertFehler(UdaFehlerSubtyp.FUNKTION,
							"Die Funktion \"" + name + "\" ist nicht definiert");
				}
			}
		}

		if (result == null) {
			result = NichtWert.NICHTWERT;
		} else if (result instanceof RueckgabeAnweisung) {
			result = ((RueckgabeAnweisung) result).getRueckgabeWert();
		}
		if ((definition != null) && (result instanceof UdaFehler)) {
			((UdaFehler) result).setFunktion(getName());
		}
		return result;
	}
}
