/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.daten;

import de.bsvrz.sys.funclib.bitctrl.interpreter.Operator;

/**
 * Die Definition der Operatoren für arithmetische Operationen.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public final class ArithmetikOperators {
	/**
	 * Multiplikation von zwei numerischen Faktoren. Werden zwei Ganzzahlen
	 * multipliziert, ist das Ergebnis eine Ganzzahl, in allen anderen Fällen
	 * eine Fliesskommazahl.
	 */
	public static final Operator MULTIPLIKATION = Operator.getOperator("*");

	/**
	 * Numerische Division. Das Ergebnis ist immer eine Fliesskommazahl. Die
	 * Division durch 0 führt auch bei Fließkommazahlen zu einem Fehler und
	 * nicht zum Wert "UNENDLICH".
	 */
	public static final Operator DIVISION = Operator.getOperator("/");

	/**
	 * Ganzzahl-Division. Die Operatoren werden zuerst durch Abschneiden der
	 * Nachkommazahlen in eine Ganzzahl umgewandelt und anschließend erfolgt die
	 * Division. Das Ergebnis ist immer eine Ganzzahl.
	 */
	public static final Operator GANZZAHL_DIVISION = Operator.getOperator("div");

	/**
	 * Rest der Ganzzahldivision. Die Operation folgt der in JAVA verwendeten
	 * Vorgehensweise. Das Ergebnis der Operation ergibt sich als:
	 *
	 * <pre>
	 *       x % y = (int)(x - (int)(x/y)*y)
	 * </pre>
	 *
	 * Das Ergebnis ist eine Ganzzahl, wenn beide Operanden eine Ganzzahl sind,
	 * in allen anderen Fällen liefert die Operation eine Fließkommazahl.
	 */
	public static final Operator GANZZAHL_REST = Operator.getOperator("modulo");

	/**
	 * Addition zweier Summanden. Das Ergebnis ist eine Ganzzahl, wenn beide
	 * Summanden Ganzzahlen sind. In allen anderen Fällen ist das Ergebnis eine
	 * Fliesskommazahl.
	 */
	public static final Operator ADDITION = Operator.getOperator("+");

	/**
	 * Subtraktion. Das Ergebnis ist eine Ganzzahl, wenn beide Operanden
	 * Ganzzahlen sind. In allen anderen Fällen ist das Ergebnis eine
	 * Fliesskommazahl.
	 */
	public static final Operator SUBTRAKTION = Operator.getOperator("-");

	/**
	 * Vergleich: Größer. Das Ergebnis ist ein Logischer Wert
	 */
	public static final Operator GROESSER = Operator.getOperator(">");

	/**
	 * Vergleich: Kleiner. Das Ergebnis ist ein Logischer Wert
	 */
	public static final Operator KLEINER = Operator.getOperator("<");

	/**
	 * Vergleich: Gleich. Das Ergebnis ist ein Logischer Wert
	 */
	public static final Operator GLEICH = Operator.getOperator("=");

	/**
	 * Vergleich: Ungleich. Das Ergebnis ist ein Logischer Wert
	 */
	public static final Operator UNGLEICH = Operator.getOperator("<>");

	/**
	 * Vergleich: Größer oder Gleich. Das Ergebnis ist ein Logischer Wert
	 */
	public static final Operator GROESSER_GLEICH = Operator.getOperator(">=");

	/**
	 * Vergleich: Kleiner oder Gleich. Das Ergebnis ist ein Logischer Wert
	 */
	public static final Operator KLEINER_GLEICH = Operator.getOperator("<=");

	/**
	 * Potenzfunktion: Das Ergebnis ist immer eine Fliesskommazahl.
	 */
	public static final Operator POTENZ = Operator.getOperator("potenz");

	/** Wurzel: Das Ergebnis ist immer eine Fliesskommazahl. */
	public static final Operator WURZEL = Operator.getOperator("wurzel");

	/**
	 * Exponentialfunktion e hoch n: Das Ergebnis ist immer eine
	 * Fliesskommazahl.
	 */
	public static final Operator EXP = Operator.getOperator("exp");

	/**
	 * Natürlicher Logarithmus: Das Ergebnis ist immer eine Fliesskommazahl.
	 */
	public static final Operator LN = Operator.getOperator("ln");

	/**
	 * Sinus: Das Ergebnis ist immer eine Fliesskommazahl.
	 */
	public static final Operator SIN = Operator.getOperator("sin");

	/**
	 * Cosinus: Das Ergebnis ist immer eine Fliesskommazahl.
	 */
	public static final Operator COS = Operator.getOperator("cos");

	/**
	 * Tangens: Das Ergebnis ist immer eine Fliesskommazahl.
	 */
	public static final Operator TAN = Operator.getOperator("tan");

	/**
	 * Minimum: Das Ergebnis entspricht dem Element mit dem minimalen Wert.
	 */
	public static final Operator MIN = Operator.getOperator("min");

	/**
	 * Maximum: Das Ergebnis entspricht dem Element mit dem maximalen Wert.
	 */
	public static final Operator MAX = Operator.getOperator("max");

	/**
	 * Absolutbetrag: Das Ergebnis entspricht dem Typ des übergbenen Arguments.
	 */
	public static final Operator ABS = Operator.getOperator("abs");

	/** Umrechnung von Grad in Bogenmass. */
	public static final Operator IN_BOGENMASS = Operator.getOperator("inBogenmass");

	/** Umrechnung von Bogenmass in Grad. */
	public static final Operator IN_GRAD = Operator.getOperator("inGrad");

	/**
	 * die unterstützten Operatoren.
	 */
	public static final Operator[] OPERATOREN = { ArithmetikOperators.MULTIPLIKATION, ArithmetikOperators.DIVISION,
			ArithmetikOperators.GANZZAHL_DIVISION, ArithmetikOperators.GANZZAHL_REST, ArithmetikOperators.ADDITION,
			ArithmetikOperators.SUBTRAKTION, ArithmetikOperators.GROESSER, ArithmetikOperators.KLEINER,
			ArithmetikOperators.GLEICH, ArithmetikOperators.UNGLEICH, ArithmetikOperators.GROESSER_GLEICH,
			ArithmetikOperators.KLEINER_GLEICH, ArithmetikOperators.POTENZ, ArithmetikOperators.EXP,
			ArithmetikOperators.LN, ArithmetikOperators.SIN, ArithmetikOperators.COS, ArithmetikOperators.TAN,
			ArithmetikOperators.MIN, ArithmetikOperators.MAX, ArithmetikOperators.ABS, ArithmetikOperators.IN_BOGENMASS,
			ArithmetikOperators.IN_GRAD, ArithmetikOperators.WURZEL };

	/** privater Konstruktor für Utility-Klasse. */
	private ArithmetikOperators() {
		super();
	}

}
