/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.daten;

import java.util.HashMap;
import java.util.Map;

import de.bsvrz.ibv.uda.interpreter.UdaInterpreter;
import de.bsvrz.ibv.uda.interpreter.ausdruck.FunktionsDefinition;
import de.bsvrz.ibv.uda.interpreter.ausdruck.Struktur;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.ObjektZugriffsFehler;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.SymbolUndefiniertFehler;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.UdaFehlerSubtyp;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Kontext;
import de.bsvrz.sys.funclib.bitctrl.interpreter.SymbolUndefiniertException;

/**
 * Allgemeines Kontextobjekt zum Verwendung innerhalb des Interpreters eines
 * Uda-Skriptes.<br>
 * Die Klasse erweitert die Klasse {@link Kontext} und implementiert die
 * Schnittstelle {@link Struktur}, um dem Zugriff auf die Kontextelemente über
 * den Elementzugriffsoperator "." zu ermöglichen.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class SkriptKontext extends Kontext implements Struktur {

	/**
	 * der verwendete Interpreter.
	 */
	private UdaInterpreter interpreter;

	/**
	 * lokale Funktionen innerhalb des Skriptkontext.
	 */
	private final Map<String, FunktionsDefinition> funktionen = new HashMap<>();

	/**
	 * erzeugt einen neuen Kontext auf Basis des übergebenen.<br>
	 * Es werden alle Elemente vom Typ {@link Kontext} in den neuen Kontext
	 * übernommen. Mit der Option vollstaendig kann eine komplette Kopie des
	 * Kontextes erzwungen werden.
	 *
	 * @param kontext
	 *            der Kontext, dessen Daten übernommen werden sollen
	 * @param vollstaendig
	 *            alle Einträge übernehmen ?
	 */
	public SkriptKontext(final SkriptKontext kontext, final boolean vollstaendig) {
		final SkriptKontext skriptKontext = kontext;
		for (final String name : skriptKontext.getVariablen()) {
			if (vollstaendig || (skriptKontext.get(name) instanceof Kontext)) {
				set(name, skriptKontext.get(name));
			}
		}
		for (final String name : kontext.funktionen.keySet()) {
			setFunktion(name, skriptKontext.getFunktion(name));
		}
		setInterpreter(kontext.getInterpreter());
	}

	/**
	 * Konstruktor.<br>
	 *
	 * @param interpreter
	 *            der Interpreter, in dem der Kontext verwendet wird
	 */
	public SkriptKontext(final UdaInterpreter interpreter) {
		super();
		this.interpreter = interpreter;
	}

	/**
	 * {@inheritDoc}.<br>
	 * Die Funktion fängt den Fehler ab, der entsteht, wenn auf eine
	 * undefinierte Variable zugegriffen wird und erzeugt stattdessen ein
	 * Uda-Fehlerobjekt.
	 *
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Kontext#get(java.lang.String)
	 */
	@Override
	public Object get(final String name) {
		Object result = null;
		try {
			result = super.get(name);
		} catch (final SymbolUndefiniertException e) {
			result = new SymbolUndefiniertFehler(UdaFehlerSubtyp.OBJEKT, e.getMessage());
		}
		return result;
	}

	/**
	 * liefert die lokale Funktion mit dem übergebenen Name.
	 *
	 * @param name
	 *            der Name
	 * @return die Funktion oder <code>null</code>, wenn keine definiert ist
	 */
	public FunktionsDefinition getFunktion(final String name) {
		return funktionen.get(name);
	}

	/**
	 * liefert den Interpreter, in dem der Kontext verwendet wird.
	 *
	 * @return den Interpreter
	 * @see #setInterpreter(UdaInterpreter)
	 */
	public UdaInterpreter getInterpreter() {
		return interpreter;
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see #setStrukturElement(String, Object)
	 */
	@Override
	public Object getStrukturElement(final String name) {
		Object ergebnis = get(name);
		if (ergebnis == null) {
			ergebnis = new ObjektZugriffsFehler(UdaFehlerSubtyp.ELEMENT, "Objekt mit dem Namen " + name
					+ " nicht im Kontext gefunden");
		}
		return ergebnis;
	}

	/**
	 * die Funktion übernimmt alle Werte aus dem übergebenen Kontext mit
	 * Ausnahme des Eintrags der angegebenen Ausnahme.
	 *
	 * @param ausnahme
	 *            die Name der Variable, die nicht aus dem übergebenen Kontext
	 *            übernommen werden soll.
	 * @param subkontext
	 *            der Kontext aus dem die datenübernommen werden
	 */
	public void reset(final String ausnahme, final SkriptKontext subkontext) {
		for (final String key : this.getVariablen()) {
			if (!key.equals(ausnahme)) {
				this.set(key, subkontext.get(key));
			}
		}
	}

	/**
	 * fügt dem Kontext eine lokale Funktion hinzu.
	 *
	 * @param name
	 *            der Name der Funktion
	 * @param funktion
	 *            die Funktion
	 */
	public void setFunktion(final String name, final FunktionsDefinition funktion) {
		funktionen.put(name, funktion);
	}

	/**
	 * setzt den Interpreter, in dem der Kontext verwendet wird.
	 *
	 * @param interpreter
	 *            festzulegender Interpreter
	 * @see #getInterpreter()
	 */
	private void setInterpreter(final UdaInterpreter interpreter) {
		this.interpreter = interpreter;
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see #getStrukturElement(String)
	 */
	@Override
	public void setStrukturElement(final String name, final Object daten) {
		set(name, daten);
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		final StringBuffer buffer = new StringBuffer("{");
		for (final String name : getVariablen()) {
			buffer.append(name);
			buffer.append('=');
			buffer.append(get(name));
		}
		buffer.append('}');
		return buffer.toString();
	}
}
