/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.daten.container;

import java.util.Collection;
import java.util.LinkedList;
import java.util.List;

import de.bsvrz.ibv.uda.interpreter.Kopierbar;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.ArgumentFehler;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.ContainerZugriffsFehler;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.UdaFehlerSubtyp;

/**
 * Repräsentation einer Liste als spezielle Ausprägung eines
 * {@link UdaContainer}.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class Liste extends UdaContainer implements Kopierbar {

	/**
	 * die Liste der enthalten Elemente.
	 */
	private final List<Object> liste = new LinkedList<>();

	/**
	 * Standardkonstruktor.<br>
	 * Es wird eine leere Liste erzeugt.
	 */
	public Liste() {
		this(null);
	}

	/**
	 * Konstruktor.<br>
	 * Die Funktion erzeugt ein neues Objekt vom Typ {@link Liste} und füllt
	 * diese mit den Daten des übergebenen {@link UdaContainer}.
	 *
	 * @param container
	 *            der Container mit den initialen Daten
	 */
	public Liste(final UdaContainer container) {
		super();
		if (container != null) {
			liste.addAll(container.getElemente());
		}
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.daten.container.UdaContainer#elementEinfuegen(int,
	 *      java.lang.Object)
	 */
	@Override
	void elementEinfuegen(final int index, final Object daten) {
		if (index < getGroesse()) {
			liste.add(index, daten);
		} else {
			liste.add(daten);
		}
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.daten.container.UdaContainer#elementEinfuegen(java.lang.String,
	 *      java.lang.Object)
	 */
	@Override
	void elementEinfuegen(final String schluessel, final Object daten) {
		throw new ArgumentFehler(UdaFehlerSubtyp.ARGUMENTE);
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.daten.container.UdaContainer#elementLoeschen(int)
	 */
	@Override
	Object elementLoeschen(final int index) {
		final Object element = getContainerElement(index);
		liste.remove(index - 1);
		return element;
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.daten.container.UdaContainer#elementLoeschen(java.lang.String)
	 */
	@Override
	Object elementLoeschen(final String schluessel) {
		return new ArgumentFehler(UdaFehlerSubtyp.ARGUMENTE);
	}

	/**
	 * {@inheritDoc}.<br>
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.Kopierbar#erzeugeKopie()
	 */
	@Override
	public Kopierbar erzeugeKopie() {
		final Liste result = new Liste();
		result.liste.addAll(liste);
		return result;
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.daten.container.Container#getContainerElement(int)
	 * @see #setContainerElement(int, Object)
	 */
	@Override
	public Object getContainerElement(final int index) {
		Object result = null;

		int zugriffsIndex = index;
		if (zugriffsIndex < 0) {
			zugriffsIndex = getGroesse() + 1 + zugriffsIndex;
		}
		if ((zugriffsIndex <= 0) || (zugriffsIndex > getGroesse())) {
			result = new ContainerZugriffsFehler(UdaFehlerSubtyp.INDEX);
		} else {
			result = liste.get(zugriffsIndex - 1);
		}
		return result;
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.daten.container.UdaContainer#getElemente()
	 */
	@Override
	public Collection<Object> getElemente() {
		return liste;
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.daten.container.UdaContainer#getGroesse()
	 */
	@Override
	public int getGroesse() {
		return liste.size();
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.daten.container.UdaContainer#getSchluessel(int)
	 */
	@Override
	String getSchluessel(final int position) {
		return Integer.toString(position);
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.daten.container.UdaContainer#getSchluesselPosition(java.lang.Object)
	 */
	@Override
	int getSchluesselPosition(final Object schluessel) {
		if (schluessel instanceof Number) {
			return ((Number) schluessel).intValue();
		}
		throw new ArgumentFehler(UdaFehlerSubtyp.ARGUMENTE);
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.daten.container.Container#setContainerElement(int,
	 *      java.lang.Object)
	 * @see #getContainerElement(int)
	 */
	@Override
	public void setContainerElement(final int index, final Object daten) {
		int zugriffsIndex = index;
		if (zugriffsIndex < 0) {
			zugriffsIndex = getGroesse() + 1 + zugriffsIndex;
		}
		if ((zugriffsIndex <= 0) || (zugriffsIndex > (getGroesse() + 1))) {
			throw new ContainerZugriffsFehler(UdaFehlerSubtyp.INDEX);
		}

		if (liste.size() < zugriffsIndex) {
			liste.add(daten);
		} else {
			liste.set(zugriffsIndex - 1, daten);
		}
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.daten.container.UdaContainer#toString()
	 */
	@Override
	public String toString() {
		return asString('{', '}');
	}
}
