/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.daten.container;

import java.util.Collection;
import java.util.WeakHashMap;

import de.bsvrz.ibv.uda.interpreter.ausdruck.Struktur;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.ArgumentFehler;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.ContainerZugriffsFehler;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.ObjektZugriffsFehler;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.UdaFehlerSubtyp;

/**
 * allgemeine Repräsentation eines Containerobjekts innerhalb eines
 * Uda-Interpreters.<br>
 * Ein Containerobjekt kann andere beliebige andere Elemente enthalten. Auf die
 * Elemente kann entweder per Index, Schlüssel oder Auswahlbedingungen
 * zugegriffen werden. Desweiteren kann auf die Elemente von Containern mit
 * Iteratoren zugegriffen werden.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public abstract class UdaContainer implements Container, Struktur {

	/**
	 * die Menge der Iteratoren, die aus dem Container erzeugt wurden und die
	 * bei Änderungen des Containerinhalts aktualisiert werdn müssen.
	 */
	private final WeakHashMap<UdaIterator, Boolean> iteratoren = new WeakHashMap<>();

	/**
	 * Standardkonstruktor.<br>
	 * Es wird ein leerer Container erzeugt.
	 */
	protected UdaContainer() {
		super();
	}

	/**
	 * fügt ein Element vor dem Element an der gegebenen Position in den
	 * Container ein.
	 *
	 * @param position
	 *            die Position
	 * @param element
	 *            das einzufügende Element
	 */
	abstract void elementEinfuegen(int position, Object element);

	/**
	 * fügt ein neues Element mit dem gegebenen Schlüssel in den Container ein.
	 *
	 * @param schluessel
	 *            der Schlüssel
	 * @param element
	 *            das einzufügende Element
	 */
	abstract void elementEinfuegen(String schluessel, Object element);

	/**
	 * löscht das Element an der angegebenen Position aus dem Container.
	 *
	 * @param position
	 *            die Position
	 * @return das gelöschte Element
	 */
	abstract Object elementLoeschen(int position);

	/**
	 * löscht das Element, das über den übergebenen Schlüssel identifiziert
	 * wird.
	 *
	 * @param schluessel
	 *            der Schlüssel
	 * @return das gelöschte Element
	 */
	abstract Object elementLoeschen(String schluessel);

	/**
	 * liefert die Position innerhalb des Containers, an dem sich das Element,
	 * welches durch den übergebenen Schlüssel beschrieben wird, befindet.
	 *
	 * @param schluessel
	 *            der Schlüssel
	 * @return die Position
	 */
	abstract int getSchluesselPosition(Object schluessel);

	/**
	 * liefert den Schlüssel für das Element an der angegebenen Position.
	 *
	 * @param position
	 *            die Position
	 * @return den Schlüssel
	 */
	abstract String getSchluessel(int position);

	/**
	 * fügt ein Element mit dem gegebenen Schlüsel in den Container ein.
	 *
	 * @param schluessel
	 *            der Schlüssel
	 * @param element
	 *            das einzufügende Objekt
	 * @return das eingefügte Objekt
	 */
	public final Object einfuegen(final Object schluessel, final Object element) {
		if (schluessel instanceof Number) {
			int position = ((Number) schluessel).intValue();
			if (position <= 0) {
				position = getGroesse() + position;
			} else {
				position--;
			}
			if (position < 0) {
				position = 0;
			}

			elementEinfuegen(position, element);
		} else if (schluessel != null) {
			elementEinfuegen(schluessel.toString(), element);
		} else {
			throw new ArgumentFehler(UdaFehlerSubtyp.ARGUMENTE);
		}
		for (final UdaIterator iterator : iteratoren.keySet()) {
			iterator.elementHinzugefuegt(getSchluesselPosition(schluessel));
		}
		return element;
	}

	/**
	 * fügt ein Element nach dem Element an der gegebenen Position in den
	 * Container ein.
	 *
	 * @param position
	 *            die Position
	 * @param element
	 *            das neue Element
	 */
	public final void anhaengen(final int position, final Object element) {
		einfuegen(position + 1, element);
	}

	/**
	 * löscht das Element mit dem angegebenen Schlüssel.
	 *
	 * @param schluessel
	 *            der Schlüssel
	 * @return das gelöschte Objekt
	 */
	public final Object loeschen(final Object schluessel) {
		final int index = getSchluesselPosition(schluessel);
		Object result = null;
		if (schluessel instanceof Number) {
			int zugriffsIndex = ((Number) schluessel).intValue();
			if (zugriffsIndex < 0) {
				zugriffsIndex = getGroesse() + 1 + zugriffsIndex;
			}
			if ((zugriffsIndex <= 0) || (zugriffsIndex > getGroesse())) {
				result = new ContainerZugriffsFehler(UdaFehlerSubtyp.INDEX);
			} else {
				result = elementLoeschen(zugriffsIndex);
			}
		} else {
			result = elementLoeschen(schluessel.toString());
		}
		for (final UdaIterator iterator : iteratoren.keySet()) {
			iterator.elementEntfernt(index);
		}

		return result;
	}

	/**
	 * liefert die Menge der Elemente, die im Container enthalten sind.
	 *
	 * @return die Elemente
	 */
	public abstract Collection<Object> getElemente();

	/**
	 * liefert die Größe des Containers.
	 *
	 * @return die Größe
	 */
	public abstract int getGroesse();

	/**
	 * liefert die Elemente des Container als kommagetrennte Zeichenkette, die
	 * in die übergebenen Zeichen für Anfang und Ende eingeschlossen ist.
	 *
	 * @param start
	 *            das Anfangszeichen
	 * @param ende
	 *            das Endezeichen
	 * @return die Zeichenkette
	 */
	public String asString(final char start, final char ende) {
		final StringBuffer buffer = new StringBuffer();
		for (final Object element : getElemente()) {
			if (buffer.length() > 0) {
				buffer.append(",");
			}
			buffer.append(element.toString());
		}
		buffer.insert(0, start);
		buffer.append(ende);
		return buffer.toString();
	}

	/**
	 * erzeugt einen {@link UdaIterator}, der auf die angegebene Position
	 * positioniert ist.
	 *
	 * @param position
	 *            die Position
	 * @return den Iterator
	 */
	public UdaIterator getUdaIterator(final Object position) {
		final UdaIterator iterator = new UdaIterator(this, getSchluesselPosition(position));
		iteratoren.put(iterator, Boolean.valueOf(true));
		return iterator;
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.ausdruck.Struktur#getStrukturElement(java.lang.String)
	 * @see #setStrukturElement(String, Object)
	 */
	@Override
	public Object getStrukturElement(final String name) {
		Object ergebnis = null;
		if ("anzahl".equals(name)) {
			ergebnis = Long.valueOf(getGroesse());
		} else {
			throw new ObjektZugriffsFehler(UdaFehlerSubtyp.ELEMENT);
		}

		return ergebnis;
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.ausdruck.Struktur#setStrukturElement(java.lang.String,
	 *      java.lang.Object)
	 * @see #getStrukturElement(String)
	 */
	@Override
	public void setStrukturElement(final String name, final Object daten) {
		throw new ArgumentFehler(UdaFehlerSubtyp.ARGUMENTE);

	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return asString('[', ']');
	}

}
