/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.daten.zeit;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.StringTokenizer;
import java.util.regex.Pattern;

import de.bsvrz.ibv.uda.interpreter.Kopierbar;
import de.bsvrz.ibv.uda.interpreter.ausdruck.Struktur;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.ArgumentFehler;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.ObjektZugriffsFehler;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.UdaFehlerSubtyp;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Repräsentation des Wertes eines Zeitstempels innerhalb des
 * Uda-Skript-Interpreters. Die INstanzen dieser Klasse implementieren die
 * Schnittstelle <i>Struktur</i> um den Zugriff auf die Elemente eines
 * Zeitstempels über Namen zu ermöglichen.<br>
 * Die folgenden Elemente können über Namen abgefragt werden:
 * <ol>
 * <li><b>tag</b> liefert den Tag des Monats im Bereich 1..31</li>
 * <li><b>monat</b> liefert den Monats im Bereich 1..12</li>
 * <li><b>jahr</b> liefert das Jahr als vierstellige Zahl</li>
 * <li><b>stunde</b> liefert die Stunde im Bereich 0..23</li>
 * <li><b>minute</b> liefert die Minute im Bereich 0..59</li>
 * <li><b>sekunde</b> liefert die Sekunde im Bereich 0..59</li>
 * <li><b>millisekunde</b> liefert die Millisekunden</li>
 * <li><b>millisekunden</b> liefert den Zeitstempel als Millisekunden seit
 * 1.1.1970 GMT</li>
 * </ol>
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class Zeitstempel implements Struktur, Comparable<Zeitstempel>, ZeitWert, Kopierbar {

	private static final Debug LOGGER = Debug.getLogger();

	/**
	 * die innerhalb des Objekts gespeicherte Zeit.
	 */
	private Calendar zeit = Calendar.getInstance();

	/**
	 * das Standardformat für die Darstellung der Zeit als Zeichenkette.
	 */
	private final DateFormat defaultFormat = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss,SSS");

	/**
	 * Konstruktor, der einen Zeitstempel mit dem aktuellen Zeitpunkt erzeugt.
	 */
	public Zeitstempel() {
		this(null);
	}

	/**
	 * Konstruktor zum Anlagen eines Zeitstempels aus einer in Millisekunden
	 * seit 1.1.1970 GMT übergebenen Zeit.
	 *
	 * @param zeitWert
	 *            der Wert
	 */
	public Zeitstempel(final long zeitWert) {
		zeit.setTimeInMillis(zeitWert);
	}

	/**
	 * Konstruktor zum Anlegen eines Zeitstempels aus einer übergebenen
	 * Zeichenkette.
	 *
	 * @param zeitAngabe
	 *            die Zeitangabe als Zeichenkette in einem zulässigem Format
	 */
	public Zeitstempel(final String zeitAngabe) {
		super();
		zeit = parseZeit(zeitAngabe);
	}

	/**
	 * verschiebt den Zeitstempel innerhalb des Objekts um die angegebene Zahl
	 * von Millisekunden.
	 *
	 * @param value
	 *            die Zeitdifferenz
	 */
	public void add(final long value) {
		zeit.setTimeInMillis(zeit.getTimeInMillis() + value);
	}

	/**
	 * vergleicht den Zeitstempel mit dem übergebenen.<br>
	 *
	 * @param o
	 *            der Vergleichswert
	 * @return das Vergleichsergebnis entsprechend der Definition in
	 *         {@link Comparable}.
	 * @see java.lang.Comparable#compareTo(java.lang.Object)
	 */
	@Override
	public int compareTo(final Zeitstempel o) {
		if (getMilliSekunden() < o.getMilliSekunden()) {
			return -1;
		}
		if (getMilliSekunden() > o.getMilliSekunden()) {
			return 1;
		}
		return 0;
	}

	@Override
	public boolean equals(final Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		final Zeitstempel other = (Zeitstempel) obj;
		if (zeit == null) {
			if (other.zeit != null) {
				return false;
			}
		} else if (!zeit.equals(other.zeit)) {
			return false;
		}
		return true;
	}

	/**
	 * {@inheritDoc}.<br>
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.Kopierbar#erzeugeKopie()
	 */
	@Override
	public Zeitstempel erzeugeKopie() {
		return new Zeitstempel(zeit.getTimeInMillis());
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = (prime * result) + ((defaultFormat == null) ? 0 : defaultFormat.hashCode());
		result = (prime * result) + ((zeit == null) ? 0 : zeit.hashCode());
		return result;
	}

	/**
	 * erzeugt eine Zeichenkette, die den Zeitstempel im übergebenen Format
	 * darstellt. Wird ein ungültiger Formatstring angegeben, erfolgt die
	 * Ausgabe im Standardformat. Folgende Platzhalter für die Definition des
	 * Formats sind erlaubt:
	 * <ol>
	 * <li><b>JJJJ</b> für die Ausgabe der vierstelligen Jahreszahl</li>
	 * <li><b>JJ</b> für die Ausgabe der zweistelligen Jahreszahl</li>
	 * <li><b>MM</b> für die Ausgabe des Monats</li>
	 * <li><b>TT</b> für die Ausgabe des Tags des Monats</li>
	 * <li><b>hh</b> für die Ausgabe der Stunde</li>
	 * <li><b>mm</b> für die Ausgabe der Minute</li>
	 * <li><b>ss</b> für die Ausgabe der Sekunde</li>
	 * <li><b>ttt</b> füür die Ausgabe der Millisekunden</li>
	 * </ol>
	 * Die Angabe der Elemente im Formatstring muss ind der angegebenen Form mit
	 * den angegebenen Stellenzahlen erfolgen. Als zusätzliche Zeichen sind '.',
	 * ':', '/' sowie das Leerzeichen erlaubt.
	 *
	 * @param format
	 *            die Beschreibung des Formats für die Ausgabe
	 * @return die Zeichenkette, die den Wert des zeitstempels repräsentiert
	 */
	public Object format(final String format) {
		String result = null;

		if (format != null) {
			final Pattern numberPattern = Pattern.compile(".*[1234567890].*");

			if (!numberPattern.matcher(format).matches()) {
				final String[] replacements = { "JJJJ", "1", "yyyy", "JJ", "2",
						"yy", "MM", "3", "MM", "TT", "4", "dd", "hh", "5",
						"HH", "mm", "6", "mm", "ss", "7", "ss", "ttt", "8",
				"SSS" };

				final StringBuilder formatBuffer = new StringBuilder(format);
				for (int idx = 0; idx < (replacements.length / 3); idx++) {
					int offset = -1;
					while ((offset = formatBuffer.indexOf(replacements[idx * 3])) >= 0) {
						formatBuffer.replace(offset, offset + replacements[idx * 3].length(),
								replacements[(idx * 3) + 1]);
					}
				}

				final Pattern symbolPattern = Pattern.compile(".*[TMJhmst].*");
				if (!symbolPattern.matcher(formatBuffer).matches()) {
					for (int idx = 0; idx < (replacements.length / 3); idx++) {
						int offset = -1;
						while ((offset = formatBuffer.indexOf(replacements[(idx * 3) + 1])) >= 0) {
							formatBuffer.replace(offset, offset + replacements[(idx * 3) + 1].length(),
									replacements[(idx * 3) + 2]);
						}
					}

					try {
						result = new SimpleDateFormat(formatBuffer.toString()).format(zeit.getTime());
					} catch (final IllegalArgumentException e) {
						LOGGER.error(e.getLocalizedMessage());
						// verwende Standardformat
					}
				}
			}
		}

		if (result == null) {
			result = toString();
		}

		return result;
	}

	/**
	 * liefert den Wert des zeitstempels als Millisekunden seit dem 1.1.1970
	 * GMT.
	 *
	 * @return den Wert
	 * @see #setMilliSekunden(long)
	 */
	@Override
	public long getMilliSekunden() {
		return zeit.getTimeInMillis();
	}

	/**
	 * liefert ein über einen Namen definiertes Element aus dem Objekt.<br>
	 * Gültige Elemente sind <i>tag</i>, <i>monat</i>, <i>jahr</i>,
	 * <i>stunde</i>, <i>minute</i>, <i>sekunde</i>, <i>millisekunde</i>,
	 * <i>millisekunden</i>.<br>
	 * {@inheritDoc}
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.ausdruck.Struktur#getStrukturElement(java.lang.String)
	 * @see #setStrukturElement(String, Object)
	 */
	@Override
	public Object getStrukturElement(final String name) {
		Object result = null;
		if ("tag".equals(name)) {
			result = Long.valueOf(zeit.get(Calendar.DAY_OF_MONTH));
		} else if ("monat".equals(name)) {
			result = Long.valueOf(zeit.get(Calendar.MONTH)) + 1L;
		} else if ("jahr".equals(name)) {
			result = Long.valueOf(zeit.get(Calendar.YEAR));
		} else if ("stunde".equals(name)) {
			result = Long.valueOf(zeit.get(Calendar.HOUR_OF_DAY));
		} else if ("minute".equals(name)) {
			result = Long.valueOf(zeit.get(Calendar.MINUTE));
		} else if ("sekunde".equals(name)) {
			result = Long.valueOf(zeit.get(Calendar.SECOND));
		} else if ("millisekunde".equals(name)) {
			result = Long.valueOf(zeit.get(Calendar.MILLISECOND));
		} else if ("millisekunden".equals(name)) {
			result = Long.valueOf(zeit.getTimeInMillis());
		} else if ("wochentag".equals(name)) {
			result = Long.valueOf(zeit.get(Calendar.DAY_OF_WEEK));
		}

		if (result == null) {
			throw new ObjektZugriffsFehler(UdaFehlerSubtyp.ELEMENT, "Das Element \"" + name + "\" kann vom Typ \""
					+ getClass().getSimpleName() + "\" nicht ermittelt werden");
		}
		return result;
	}

	/**
	 * analysiert die übergebene Zeichenkette und erzeugt daraus ein Datum.
	 *
	 * @param zeitAngabe
	 *            die Zeitangabe als zeichenkette
	 * @return das Datum
	 */
	private GregorianCalendar parseZeit(final String zeitAngabe) {
		final GregorianCalendar now = new GregorianCalendar();
		GregorianCalendar cal = null;
		if (zeitAngabe != null) {
			cal = new GregorianCalendar(now.get(Calendar.YEAR), now.get(Calendar.MONTH),
					now.get(Calendar.DAY_OF_MONTH));
			int tag = cal.get(Calendar.DAY_OF_MONTH);
			int monat = cal.get(Calendar.MONTH);
			int jahr = cal.get(Calendar.YEAR);
			int stunde = cal.get(Calendar.HOUR_OF_DAY);
			int minute = cal.get(Calendar.MINUTE);
			int sekunde = cal.get(Calendar.SECOND);
			int millisekunde = cal.get(Calendar.MILLISECOND);

			String datePart = null;
			String timePart = null;
			String milliPart = null;

			StringTokenizer tokenizer = new StringTokenizer(zeitAngabe, " ");
			if ((tokenizer.countTokens() > 0) && (tokenizer.countTokens() <= 2)) {
				final String token = tokenizer.nextToken();
				if (tokenizer.hasMoreTokens()) {
					datePart = token.trim();
					timePart = tokenizer.nextToken().trim();
				} else {
					if (token.contains(".")) {
						datePart = token;
					} else {
						timePart = token;
					}
				}
			}

			if (datePart != null) {
				tokenizer = new StringTokenizer(datePart, ".");
				if (tokenizer.countTokens() != 3) {
					throw new ArgumentFehler(UdaFehlerSubtyp.ARGUMENTE);
				}
				tag = Integer.parseInt(tokenizer.nextToken());
				cal.set(Calendar.DAY_OF_MONTH, tag);
				monat = Integer.parseInt(tokenizer.nextToken()) - 1;
				cal.set(Calendar.MONTH, monat);
				final String token = tokenizer.nextToken();
				jahr = Integer.parseInt(token);
				if ((token.length() != 2) && (token.length() != 4)) {
					throw new ArgumentFehler(UdaFehlerSubtyp.ARGUMENTE);
				}
				if (token.length() == 2) {
					jahr += 2000;
				}
				cal.set(Calendar.YEAR, jahr);
			}

			if (timePart != null) {
				tokenizer = new StringTokenizer(timePart, ",");
				if ((tokenizer.countTokens() > 0) && (tokenizer.countTokens() <= 2)) {
					final String token = tokenizer.nextToken();
					if (tokenizer.hasMoreTokens()) {
						timePart = token.trim();
						milliPart = tokenizer.nextToken().trim();
					}
				}

				tokenizer = new StringTokenizer(timePart, ":");
				if ((tokenizer.countTokens() <= 0) || (tokenizer.countTokens() > 3)) {
					throw new ArgumentFehler(UdaFehlerSubtyp.ARGUMENTE);
				}
				int step = 0;
				while (tokenizer.hasMoreTokens()) {
					final String token = tokenizer.nextToken();
					switch (step) {
					case 0:
						stunde = Integer.parseInt(token);
						cal.set(Calendar.HOUR_OF_DAY, stunde);
						break;
					case 1:
						if (token.length() != 2) {
							throw new ArgumentFehler(UdaFehlerSubtyp.ARGUMENTE,
									"Die Minutenangabe muss zweistellig erfolgen");
						}
						minute = Integer.parseInt(token);
						cal.set(Calendar.MINUTE, minute);
						break;
					case 2:
						if (token.length() != 2) {
							throw new ArgumentFehler(UdaFehlerSubtyp.ARGUMENTE,
									"Die Sekundenangabe muss zweistellig erfolgen");
						}
						sekunde = Integer.parseInt(token);
						cal.set(Calendar.SECOND, sekunde);
						break;
					default:
						// es werden keine weiteren Elemente erwartet
						break;
					}
					step++;
				}
				if (milliPart != null) {
					millisekunde = Integer.parseInt(milliPart);
					if (milliPart.length() > 3) {
						throw new ArgumentFehler(UdaFehlerSubtyp.ARGUMENTE);
					}
					for (int idx = milliPart.length(); idx < 3; idx++) {
						millisekunde *= 10;
					}
					cal.set(Calendar.MILLISECOND, millisekunde);
				}
			}

			if (cal.get(Calendar.DAY_OF_MONTH) != tag) {
				throw new ArgumentFehler(UdaFehlerSubtyp.ARGUMENTE);
			}
			if (cal.get(Calendar.MONTH) != monat) {
				throw new ArgumentFehler(UdaFehlerSubtyp.ARGUMENTE);
			}
			if (cal.get(Calendar.YEAR) != jahr) {
				throw new ArgumentFehler(UdaFehlerSubtyp.ARGUMENTE);
			}
			if (cal.get(Calendar.HOUR_OF_DAY) != stunde) {
				throw new ArgumentFehler(UdaFehlerSubtyp.ARGUMENTE);
			}
			if (cal.get(Calendar.MINUTE) != minute) {
				throw new ArgumentFehler(UdaFehlerSubtyp.ARGUMENTE);
			}
		} else {
			cal = now;
		}

		return cal;
	}

	/**
	 * setzt den Wert des Zeitstempels auf den in Millisekunden seit 1.1.1970
	 * GMT angegebenen.
	 *
	 * @param wert
	 *            der Wert
	 * @see #getMilliSekunden()
	 */
	@Override
	public void setMilliSekunden(final long wert) {
		zeit.setTimeInMillis(wert);
	}

	/**
	 * setzt ein über einen Namen definiertes Element innerhalb des
	 * Zeitstempels.<br>
	 * Gültige Elemente sind <i>tag</i>, <i>monat</i>, <i>jahr</i>,
	 * <i>stunde</i>, <i>minute</i>, <i>sekunde</i>, <i>millisekunde</i>,
	 * <i>millisekunden</i>.<br>
	 * Die Werte werden nicht auf ihren Bereich überprüft. Wird ein Zahlenwert
	 * übergeben, der den jeweiligen Bereich überschreitet erfolgt ein Ãberlauf
	 * auf das jeweilige benachbarte Element.<br>
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.ausdruck.Struktur#setStrukturElement(java.lang.String,
	 *      java.lang.Object)
	 * @see #getStrukturElement(String)
	 */
	@Override
	public void setStrukturElement(final String name, final Object daten) {
		if (daten instanceof Number) {
			final int value = ((Number) daten).intValue();

			if ("tag".equals(name)) {
				zeit.set(Calendar.DAY_OF_MONTH, value);
			} else if ("monat".equals(name)) {
				zeit.set(Calendar.MONTH, value - 1);
			} else if ("jahr".equals(name)) {
				zeit.set(Calendar.YEAR, value);
			} else if ("stunde".equals(name)) {
				zeit.set(Calendar.HOUR_OF_DAY, value);
			} else if ("minute".equals(name)) {
				zeit.set(Calendar.MINUTE, value);
			} else if ("sekunde".equals(name)) {
				zeit.set(Calendar.SECOND, value);
			} else if ("millisekunde".equals(name)) {
				zeit.set(Calendar.MILLISECOND, value);
			} else if ("millisekunden".equals(name)) {
				zeit.setTimeInMillis(((Number) daten).longValue());
			}
		}
	}

	/**
	 * gibt den Zeitstempel im Standardformat als Zeichenkette aus.<br>
	 * {@inheritDoc}.
	 *
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return defaultFormat.format(zeit.getTime());
	}
}
