/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.daten.fuzzy;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import de.bsvrz.iav.fuzzylib.fuzzylib.FuzzySet;
import de.bsvrz.iav.fuzzylib.fuzzylib.Term;
import de.bsvrz.iav.fuzzylib.interpreter.logik.FuzzyLogikHandler;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.ArgumentFehler;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.Fehler;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.UdaFehlerSubtyp;
import de.bsvrz.sys.funclib.bitctrl.interpreter.HandlerValidation;
import de.bsvrz.sys.funclib.bitctrl.interpreter.InterpreterException;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Operator;
import de.bsvrz.sys.funclib.bitctrl.interpreter.logik.LogischerWert;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Handler für die Ausführung von Fuzzy-Opertaionen innerhalb eines
 * UdaInterpreter.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class UdaFuzzyHandler extends FuzzyLogikHandler {

	private static final Debug LOGGER = Debug.getLogger();

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Handler#getHandledOperators()
	 */
	@Override
	public Operator[] getHandledOperators() {
		return UdaFuzzyOperators.OPERATOREN;
	}

	/**
	 * konvertiert die Liste der übergebenen Operanden in Objekte com Typ
	 * {@link LogischerWert}.
	 *
	 * @param operanden
	 *            die zu konvertierenden Objekte
	 * @return die logischen Werte
	 */
	private List<Object> konvertiereInLogischeWerte(final List<? extends Object> operanden) {
		final List<Object> result = new ArrayList<>();

		for (final Object operand : operanden) {
			if (operand instanceof Number) {
				try {
					result.add(new LogischerWert(((Number) operand).floatValue()));
				} catch (@SuppressWarnings("unused") final InterpreterException e) {
					result.add(operand);
				}
			} else {
				result.add(operand);
			}
		}
		return result;
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Handler#perform(de.bsvrz.sys.funclib.bitctrl.interpreter.Operator,
	 *      java.util.List)
	 */
	@Override
	public Object perform(final Operator operator, final List<Object> operanden) {

		final HandlerValidation validation = validiereHandler(operator, operanden);
		if (!validation.isValid()) {
			if (!validation.isRichtigeAnzahl()) {
				throw new ArgumentFehler(UdaFehlerSubtyp.ANZAHL);
			}
			if (!validation.isRichtigerTyp()) {
				throw new ArgumentFehler(UdaFehlerSubtyp.ARGUMENTE);
			}
			throw new ArgumentFehler(UdaFehlerSubtyp.UNBEKANNT);
		}

		Object result = null;
		try {
			if (UdaFuzzyOperators.LING_TERM_KONSTRUKTOR.equals(operator)) {
				final String name = (String) operanden.get(0);
				final Number unten = (Number) operanden.get(1);
				final Number oben = (Number) operanden.get(2);
				String desc = null;
				if (operanden.size() > 3) {
					desc = (String) operanden.get(3);
				}
				result = new UdaLinguistischerTerm(name, unten.intValue(), oben.intValue(), desc);
			} else if (UdaFuzzyOperators.LING_VAR_KONSTRUKTOR.equals(operator)) {
				final String name = (String) operanden.get(0);
				final Number unten = (Number) operanden.get(1);
				final Number oben = (Number) operanden.get(2);
				final Collection<UdaLinguistischerTerm> terme = new ArrayList<>();
				for (int idx = 3; idx < operanden.size(); idx++) {
					terme.add((UdaLinguistischerTerm) operanden.get(idx));
				}
				result = new UdaLinguistischeVariable(name, unten.intValue(), oben.intValue());
				for (final FuzzySet set : terme) {
					((UdaLinguistischeVariable) result).addFuzzySet(set);
				}
			} else if (UdaFuzzyOperators.FUZZYFIZIERE.equals(operator)) {
				final UdaLinguistischeVariable var = (UdaLinguistischeVariable) operanden.get(0);
				if (operanden.size() > 1) {
					final Number wert = (Number) operanden.get(1);
					result = new UdaFuzzyWert(var.fuzzyfiziere(wert.intValue()));
				} else {
					result = new UdaFuzzyWert(var.fuzzyfiziere());
				}
			} else if (UdaFuzzyOperators.DEFUZZYFIZIERE.equals(operator)) {
				final UdaLinguistischeVariable var = (UdaLinguistischeVariable) operanden.get(0);
				final UdaFuzzyWert wert = (UdaFuzzyWert) operanden.get(1);
				result = var.defuzzyfiziere(wert.getVariable());
				if (result == null) {
					throw new Fehler("Die Defuzzyfizierung konnte nicht ausgeführt werden");
				}
			} else if (UdaFuzzyOperators.FUZZYWERT.equals(operator)) {
				final UdaLinguistischeVariable var = (UdaLinguistischeVariable) operanden.get(0);
				result = new UdaFuzzyWert(var.fuzzyfiziere());
			} else if (UdaFuzzyOperators.ZUGEHOERIGKEIT.equals(operator)) {
				final UdaFuzzyWert wert = (UdaFuzzyWert) operanden.get(0);
				final String name = (String) operanden.get(1);
				if (operanden.size() <= 2) {
					final Term term = (Term) wert.getStrukturElement(name);
					if (term == null) {
						throw new ArgumentFehler(UdaFehlerSubtyp.ARGUMENTE,
								"Es existiert kein Term mit dem Namen \"" + name + "\"");
					}
					result = Double.valueOf(term.getZugehoerigkeit());
				} else {
					final Number zahl = (Number) operanden.get(2);
					wert.setStrukturElement(name, zahl);
					result = wert;
				}
			} else if (UdaFuzzyOperators.KOMPLEMENT.equals(operator)) {
				result = super.perform(FuzzyLogikHandler.KOMPLEMENT, konvertiereInLogischeWerte(operanden));
			} else if (UdaFuzzyOperators.GAMMA.equals(operator)) {
				result = super.perform(FuzzyLogikHandler.GAMMA_OPERATOR, konvertiereInLogischeWerte(operanden));
			} else if (UdaFuzzyOperators.MAXIMUM.equals(operator)) {
				result = super.perform(FuzzyLogikHandler.MAXIMUM, operanden);
			} else if (UdaFuzzyOperators.MINIMUM.equals(operator)) {
				result = super.perform(FuzzyLogikHandler.MINIMUM, operanden);
			} else if (UdaFuzzyOperators.ALGSUMME.equals(operator)) {
				result = super.perform(FuzzyLogikHandler.ALGEBRAISCHE_SUMME, operanden);
			} else if (UdaFuzzyOperators.ALGPRODUKT.equals(operator)) {
				result = super.perform(FuzzyLogikHandler.ALGEBRAISCHES_PRODUKT, operanden);
			}
		} catch (final ClassCastException e) {
			final String message = "Der Operator \"" + operator.getSymbol() + "\" ist auf den Typ \""
					+ operanden.get(0).getClass() + "\" nicht anwendbar";
			UdaFuzzyHandler.LOGGER.error(message, e);
			throw new Fehler(message);
		}
		return result;
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Handler#validiereHandler(de.bsvrz.sys.funclib.bitctrl.interpreter.Operator,
	 *      java.util.List)
	 */
	@Override
	public HandlerValidation validiereHandler(final Operator operator, final List<? extends Object> operanden) {
		HandlerValidation result = null;

		boolean anzahlOk = false;
		boolean typOk = false;

		if (UdaFuzzyOperators.LING_TERM_KONSTRUKTOR.equals(operator)) {
			anzahlOk = true;
			typOk = true;
		} else if (UdaFuzzyOperators.LING_VAR_KONSTRUKTOR.equals(operator)) {
			anzahlOk = true;
			typOk = true;
		} else if (UdaFuzzyOperators.FUZZYFIZIERE.equals(operator)) {
			anzahlOk = true;
			typOk = true;
		} else if (UdaFuzzyOperators.DEFUZZYFIZIERE.equals(operator)) {
			anzahlOk = true;
			typOk = true;
		} else if (UdaFuzzyOperators.FUZZYWERT.equals(operator)) {
			anzahlOk = true;
			typOk = true;
		} else if (UdaFuzzyOperators.ZUGEHOERIGKEIT.equals(operator)) {
			anzahlOk = true;
			typOk = true;
		} else if (UdaFuzzyOperators.MAXIMUM.equals(operator)) {
			result = super.validiereHandler(FuzzyLogikHandler.MAXIMUM, operanden);
		} else if (UdaFuzzyOperators.MINIMUM.equals(operator)) {
			result = super.validiereHandler(FuzzyLogikHandler.MINIMUM, operanden);
		} else if (UdaFuzzyOperators.ALGSUMME.equals(operator)) {
			result = super.validiereHandler(FuzzyLogikHandler.ALGEBRAISCHE_SUMME, operanden);
		} else if (UdaFuzzyOperators.ALGPRODUKT.equals(operator)) {
			result = super.validiereHandler(FuzzyLogikHandler.ALGEBRAISCHES_PRODUKT, operanden);
		} else if (UdaFuzzyOperators.GAMMA.equals(operator)) {
			result = super.validiereHandler(FuzzyLogikHandler.GAMMA_OPERATOR, konvertiereInLogischeWerte(operanden));
		} else if (UdaFuzzyOperators.KOMPLEMENT.equals(operator)) {
			result = super.validiereHandler(FuzzyLogikHandler.KOMPLEMENT, konvertiereInLogischeWerte(operanden));
		} else {
			result = super.validiereHandler(operator, operanden);
		}

		if (result == null) {
			result = new HandlerValidation(anzahlOk, typOk);
		}

		return result;
	}
}
